# coding: utf-8
from future.builtins import str
import datetime
import sys
import traceback

from celery import states

from aio_client.base.exceptions import AioClientException
from aio_client.provider.api import get_requests, set_error_requests
from concentrator.smev3 import kinder_conc
from kinder.core.async.tasks import PeriodicAsyncTask


class Smev3PeriodicAsyncTask(PeriodicAsyncTask):
    """Периодическая таска, которая проверяет наличие в клиенте заявок
    с видом сведений message_type, запускает по ним формирование
    и отправку ответа
    """
    message_type = ''
    description = ''
    stop_executing = False
    run_every = None
    LOG_TIME_FORMAT = "%d.%m.%Y %H:%M"
    helper_function = None
    abstract = True

    def run(self, *args, **kwargs):
        super(Smev3PeriodicAsyncTask, self).run(*args, **kwargs)
        values = {
            u"Время начала": datetime.datetime.now(
            ).strftime(self.LOG_TIME_FORMAT),
        }
        self.set_progress(
            progress=u"Получаем запросы %s" % self.message_type,
            values=values
        )
        result = get_requests(self.message_type)
        values[u'Кол-во сообщений'] = str(len(result))
        error_list = []
        for message in result:
            values[u'Сообщение %s' % message["origin_message_id"]] = (
                u'Отправка ответа в СМЭВ')
            try:
                self.helper_function(message=message)
            except kinder_conc.GDSParseError as exc:
                error_list.append(message["origin_message_id"])
                values[u'Ответ на сообщение %s не удалось отправить'
                       % message["origin_message_id"]] = (
                    u'ошибка при разборе входящего сообщения - %s' % str(exc))
                continue
            except AioClientException as exc:
                error_list.append(message["origin_message_id"])
                values[u'Ответ на сообщение %s не удалось отправить'
                       % message["origin_message_id"]] = (
                    u'%s (код ошибки - %s)' % (exc.message, exc.code))
                continue
            except Exception:
                error_list.append(message["origin_message_id"])
                values[u'Ответ на сообщение %s не удалось отправить'
                       % message["origin_message_id"]] = (
                    '\n'.join(traceback.format_exception(*sys.exc_info())))
                continue

            values[u'Сообщение %s' % message["origin_message_id"]] = (
                u'Отправка ответа в СМЭВ прошла успешно')

        set_error_requests(error_list)
        if error_list:
            values[u'Кол-во сообщений, по которым не удалось отправить ответ'] = (
                    str(len(error_list)))
        values[u"Время окончания"] = datetime.datetime.now(
            ).strftime(self.LOG_TIME_FORMAT)
        self.set_progress(
            progress=u'Завершено',
            task_state=states.SUCCESS,
            values=values)

        return self.state
